# -*- coding: iso-8859-1 -*-

# getPageInfoVersionWithAttributes.py --- Plugin XML-RPC command for MoinMoin getting advanced page information

# $Id: getPageInfoVersionWithAttributes.py,v 1.5 2006/05/28 12:16:31 stefan Exp $

# Copyright 2006 Stefan Merten <smerten@oekonux.de>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import time, xmlrpclib

from MoinMoin import user, wikiutil
from MoinMoin.Page import Page
from MoinMoin.logfile import editlog

###############################################################################
###############################################################################

def execute(xmlRpc, pagename, rev):
    """
    Callback to execute the functionality of this plugin.

    Returns information for a page with extended attributes.

    @param xmlRpc: Describes the local execution environment. While all other
                   parameters come from the remote caller this one is added by
                   the local execution environment.
    @type xmlRpc: XmlRpcBase

    @param pagename: The name of the page to write.
    @type pagename: unicode | str

    @param rev: The version to return information for or C{0} for the most
                recent version.
    @type rev: int

    @return: A C{Fault} for non-existing or unreadable pages or the attributes
             for this page version. See
             http://www.merten-home.de/FreeSoftware/remoteMoin/manual.html#attributes
             for the rationale of the attributes.
    @rtype: Fault |
            { 'name': str | unicode,
              'lastModified': xmlRpc.DateTime,
              'author': str | unicode,
              'version': int,
              'comment': str | unicode,
              'action': str,
              'extra': str,
              'delete': xmlrpclib.Boolean,
              'renameOrResurrect': xmlrpclib.Boolean,
              'renamedFrom': str | unicode, }
    """
    pagename = xmlRpc._instr(pagename)
    if not xmlRpc.request.user.may.read(pagename):
        return xmlrpclib.Fault(1, "You are not allowed to read this page")
    if not rev:
        page = Page(xmlRpc.request, pagename)
        rev = page.get_real_rev()
    else:
        page = Page(xmlRpc.request, pagename, rev=rev)
    if not page.isStandardPage():
        return xmlRpc.noSuchPageFault()

    edit_log = editlog.EditLog(xmlRpc.request, rootpagename=pagename)
    logs = [ log
             for log in edit_log
             if int(log.rev) == rev ]
    if not logs:
        # Revision does not appear in the logs - strange
        return xmlRpc.noSuchPageFault()
    log = logs[0]

    attributes = { }

    attributes['name'] = xmlRpc._outstr(log.pagename)

    gmtuple = tuple(time.gmtime(wikiutil.version2timestamp(log.ed_time_usecs)))
    attributes['lastModified'] = xmlrpclib.DateTime(gmtuple)

    author_str = log.hostname
    if log.userid:
        userdata = user.User(xmlRpc.request, log.userid)
        if userdata.name:
            author_str = userdata.name
    attributes['author'] = xmlRpc._outstr(author_str)

    attributes['version'] = int(log.rev)

    attributes['comment'] = xmlRpc._outstr(log.comment)

    attributes['action'] = log.action

    attributes['extra'] = log.extra

    attributes['delete'] = xmlrpclib.Boolean(not page.exists())

    # This condition marks a rename or resurrection
    renameOrResurrect = (attributes['action'] == 'SAVENEW'
                         and attributes['version'] > 1)
                        
    attributes['renameOrResurrect'] = xmlrpclib.Boolean(renameOrResurrect)

    if (renameOrResurrect
        and Page(xmlRpc.request, pagename, rev=rev - 1).exists()):
        # Try to determine the source page from the comment generated by the
        # `RenamePage' action
        renameLine = page.getlines()[0]
        fixedString = u"## page was renamed from "
        if renameLine.startswith(fixedString):
            attributes['renamedFrom'] = renameLine[len(fixedString):]
        else:
            xmlRpc.request.log(u"getPageInfoVersionWithAttributes: Renamed page does not start with `"
                               + fixedString +"'")

    return attributes
