# -*- coding: iso-8859-1 -*-

# getRecentChangesWithAttributes.py --- Plugin XML-RPC command for MoinMoin getting advanced recent changes

# $Id: getRecentChangesWithAttributes.py,v 1.6 2006/05/28 12:16:31 stefan Exp $

# Copyright 2006 Stefan Merten <smerten@oekonux.de>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import time, xmlrpclib

from MoinMoin import user, wikiutil
from MoinMoin.Page import Page
from MoinMoin.logfile import editlog

###############################################################################
###############################################################################

def execute(xmlRpc, date, pagenames):
    """
    Callback to execute the functionality of this plugin.

    Gets recent changes of all or the given pages and returns extended
    attributes.

    @param xmlRpc: Describes the local execution environment. While all other
                   parameters come from the remote caller this one is added by
                   the local execution environment.
    @type xmlRpc: XmlRpcBase

    @param date: The time since when recent changes are wanted.
    @type: xmlrpclib.DateTime

    @param pagenames: The names of the pages to get recent changes from or
                      empty for all (readable) pages.
    @type: [ unicode | str, ..., ]

    @return: List of results. Each result is either an attribute list for all
             the changes (possibly empty for no changes in the period) or a
             C{Fault} for non-existing or unreadable pages. See
             http://www.merten-home.de/FreeSoftware/remoteMoin/manual.html#attributes
             for the rationale of the attributes.
    @rtype: [ Fault |
              [ { 'name': str | unicode,
                  'lastModified': xmlRpc.DateTime,
                  'author': str | unicode,
                  'version': int,
                  'comment': str | unicode,
                  'action': str,
                  'extra': str,
                  'renameOrResurrect': xmlrpclib.Boolean, }
                ..., ],
              ..., ]
    """
    # This is taken from `wikirpc.XmlRpcBase.xmlrpc_getRecentChanges` and
    # modified.

    results = [ ]
    pagenames = [ xmlRpc._instr(pagename)
                  for pagename in pagenames ]
    if not pagenames:
        pagenames = [ None, ]

    for pagename in pagenames:
        result = [ ]
        if pagename:
            if not xmlRpc.request.user.may.read(pagename):
                result = xmlrpclib.Fault(1, "You are not allowed to read this page")
            elif not Page(xmlRpc.request, pagename).isStandardPage():
                result = xmlRpc.noSuchPageFault()
            if result:
                results.append(result)
                continue

        edit_log = editlog.EditLog(xmlRpc.request, rootpagename=pagename)
        for log in edit_log.reverse():
            attributes = { }

            # Skip pages which may not be read
            if not xmlRpc.request.user.may.read(log.pagename):
                continue
            attributes['name'] = xmlRpc._outstr(log.pagename)

            gmtuple = tuple(time.gmtime(wikiutil.version2timestamp(log.ed_time_usecs)))
            attributes['lastModified'] = xmlrpclib.DateTime(gmtuple)
            # Skip remaining entries because they are all older
            if attributes['lastModified'] < date:
                break

            author_str = log.hostname
            if log.userid:
                userdata = user.User(xmlRpc.request, log.userid)
                if userdata.name:
                    author_str = userdata.name
            attributes['author'] = xmlRpc._outstr(author_str)

            attributes['version'] = int(log.rev)

            attributes['comment'] = xmlRpc._outstr(log.comment)

            attributes['action'] = log.action

            attributes['extra'] = log.extra

            # This condition marks a rename or a resurrection
            renameOrResurrect = (attributes['action'] == 'SAVENEW'
                                 and attributes['version'] > 1)
            attributes['renameOrResurrect'] = xmlrpclib.Boolean(renameOrResurrect)

            result.append(attributes)
        results.append(result)

    return results
